@@ -1,39 +1,38 @@
 # plugin/plugin_base.py
-# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
+# mypy: ignore-errors
+
+
+from __future__ import annotations
+
+import abc
+import configparser
+import logging
+import os
+import re
+import sys
+from typing import Any
+
+from sqlalchemy.testing import asyncio
 
 """Testing extensions.
 
 this module is designed to work as a testing-framework-agnostic library,
 created so that multiple test frameworks can be supported at once
 (mostly so that we can migrate to new ones). The current target
-is py.test.
+is pytest.
 
 """
 
-from __future__ import absolute_import
-
-import abc
-import re
-import sys
-
-
-py3k = sys.version_info >= (3, 0)
-
-if py3k:
-    import configparser
-
-    ABC = abc.ABC
-else:
-    import ConfigParser as configparser
-    import collections as collections_abc  # noqa
-
-    class ABC(object):
-        __metaclass__ = abc.ABCMeta
+# flag which indicates we are in the SQLAlchemy testing suite,
+# and not that of Alembic or a third party dialect.
+bootstrapped_as_sqlalchemy = False
 
+log = logging.getLogger("sqlalchemy.testing.plugin_base")
 
 # late imports
 fixtures = None
@@ -41,6 +40,7 @@ engines = None
 exclusions = None
 warnings = None
 profiling = None
+provision = None
 assertions = None
 requirements = None
 config = None
@@ -48,7 +48,6 @@ testing = None
 util = None
 file_config = None
 
-
 logging = None
 include_tags = set()
 exclude_tags = set()
@@ -59,21 +58,21 @@ def setup_options(make_option):
     make_option(
         "--log-info",
         action="callback",
-        type="string",
+        type=str,
         callback=_log,
         help="turn on info logging for <LOG> (multiple OK)",
     )
     make_option(
         "--log-debug",
         action="callback",
-        type="string",
+        type=str,
         callback=_log,
         help="turn on debug logging for <LOG> (multiple OK)",
     )
     make_option(
         "--db",
         action="append",
-        type="string",
+        type=str,
         dest="db",
         help="Use prefab database uri. Multiple OK, "
         "first one is run by default.",
@@ -87,10 +86,19 @@ def setup_options(make_option):
     make_option(
         "--dburi",
         action="append",
-        type="string",
+        type=str,
         dest="dburi",
         help="Database uri.  Multiple OK, " "first one is run by default.",
     )
+    make_option(
+        "--dbdriver",
+        action="append",
+        type=str,
+        dest="dbdriver",
+        help="Additional database drivers to include in tests.  "
+        "These are linked to the existing database URLs by the "
+        "provisioning system.",
+    )
     make_option(
         "--dropfirst",
         action="store_true",
@@ -98,22 +106,54 @@ def setup_options(make_option):
         help="Drop all tables in the target database first",
     )
     make_option(
-        "--backend-only",
+        "--disable-asyncio",
         action="store_true",
-        dest="backend_only",
-        help="Run only tests marked with __backend__ or __sparse_backend__",
+        help="disable test / fixtures / provisoning running in asyncio",
+    )
+    make_option(
+        "--backend-only",
+        action="callback",
+        zeroarg_callback=_set_tag_include("backend"),
+        help=(
+            "Run only tests marked with __backend__ or __sparse_backend__; "
+            "this is now equivalent to the pytest -m backend mark expression"
+        ),
     )
     make_option(
         "--nomemory",
-        action="store_true",
-        dest="nomemory",
-        help="Don't run memory profiling tests",
+        action="callback",
+        zeroarg_callback=_set_tag_exclude("memory_intensive"),
+        help="Don't run memory profiling tests; "
+        "this is now equivalent to the pytest -m 'not memory_intensive' "
+        "mark expression",
     )
     make_option(
-        "--postgresql-templatedb",
-        type="string",
-        help="name of template database to use for PostgreSQL "
-        "CREATE DATABASE (defaults to current database)",
+        "--notimingintensive",
+        action="callback",
+        zeroarg_callback=_set_tag_exclude("timing_intensive"),
+        help="Don't run timing intensive tests; "
+        "this is now equivalent to the pytest -m 'not timing_intensive' "
+        "mark expression",
+    )
+    make_option(
+        "--nomypy",
+        action="callback",
+        zeroarg_callback=_set_tag_exclude("mypy"),
+        help="Don't run mypy typing tests; "
+        "this is now equivalent to the pytest -m 'not mypy' mark expression",
+    )
+    make_option(
+        "--profile-sort",
+        type=str,
+        default="cumulative",
+        dest="profilesort",
+        help="Type of sort for profiling standard output",
+    )
+    make_option(
+        "--profile-dump",
+        type=str,
+        dest="profiledump",
+        help="Filename where a single profile run will be dumped",
     )
     make_option(
         "--low-connections",
@@ -124,47 +164,33 @@ def setup_options(make_option):
     )
     make_option(
         "--write-idents",
-        type="string",
+        type=str,
         dest="write_idents",
         help="write out generated follower idents to <file>, "
         "when -n<num> is used",
     )
-    make_option(
-        "--reversetop",
-        action="store_true",
-        dest="reversetop",
-        default=False,
-        help="Use a random-ordering set implementation in the ORM "
-        "(helps reveal dependency issues)",
-    )
     make_option(
         "--requirements",
         action="callback",
-        type="string",
+        type=str,
         callback=_requirements_opt,
         help="requirements class for testing, overrides setup.cfg",
     )
-    make_option(
-        "--with-cdecimal",
-        action="store_true",
-        dest="cdecimal",
-        default=False,
-        help="Monkeypatch the cdecimal library into Python 'decimal' "
-        "for all tests",
-    )
     make_option(
         "--include-tag",
         action="callback",
         callback=_include_tag,
-        type="string",
-        help="Include tests with tag <tag>",
+        type=str,
+        help="Include tests with tag <tag>; "
+        "legacy, use pytest -m 'tag' instead",
     )
     make_option(
         "--exclude-tag",
         action="callback",
         callback=_exclude_tag,
-        type="string",
-        help="Exclude tests with tag <tag>",
+        type=str,
+        help="Exclude tests with tag <tag>; "
+        "legacy, use pytest -m 'not tag' instead",
     )
     make_option(
         "--write-profiles",
@@ -180,6 +206,33 @@ def setup_options(make_option):
         default=False,
         help="Unconditionally write/update profiling data.",
     )
+    make_option(
+        "--dump-pyannotate",
+        type=str,
+        dest="dump_pyannotate",
+        help="Run pyannotate and dump json info to given file",
+    )
+    make_option(
+        "--mypy-extra-test-path",
+        type=str,
+        action="append",
+        default=[],
+        dest="mypy_extra_test_paths",
+        help="Additional test directories to add to the mypy tests. "
+        "This is used only when running mypy tests. Multiple OK",
+    )
+    # db specific options
+    make_option(
+        "--postgresql-templatedb",
+        type=str,
+        help="name of template database to use for PostgreSQL "
+        "CREATE DATABASE (defaults to current database)",
+    )
+    make_option(
+        "--oracledb-thick-mode",
+        action="store_true",
+        help="enables the 'thick mode' when testing with oracle+oracledb",
+    )
 
 
 def configure_follower(follower_ident):
@@ -199,15 +252,9 @@ def memoize_important_follower_config(dict_):
 
     This invokes in the parent process after normal config is set up.
 
-    This is necessary as py.test seems to not be using forking, so we
-    start with nothing in memory, *but* it isn't running our argparse
-    callables, so we have to just copy all of that over.
+    Hook is currently not used.
 
     """
-    dict_["memoized_config"] = {
-        "include_tags": include_tags,
-        "exclude_tags": exclude_tags,
-    }
 
 
 def restore_important_follower_config(dict_):
@@ -215,16 +262,17 @@ def restore_important_follower_config(dict_):
 
     This invokes in the follower process.
 
+    Hook is currently not used.
+
     """
-    global include_tags, exclude_tags
-    include_tags.update(dict_["memoized_config"]["include_tags"])
-    exclude_tags.update(dict_["memoized_config"]["exclude_tags"])
 
 
-def read_config():
+def read_config(root_path):
     global file_config
     file_config = configparser.ConfigParser()
-    file_config.read(["setup.cfg", "test.cfg"])
+    file_config.read(
+        [str(root_path / "setup.cfg"), str(root_path / "test.cfg")]
+    )
 
 
 def pre_begin(opt):
@@ -246,12 +294,12 @@ def post_begin():
         fn(options, file_config)
 
     # late imports, has to happen after config.
-    global util, fixtures, engines, exclusions, assertions
+    global util, fixtures, engines, exclusions, assertions, provision
     global warnings, profiling, config, testing
     from sqlalchemy import testing  # noqa
     from sqlalchemy.testing import fixtures, engines, exclusions  # noqa
     from sqlalchemy.testing import assertions, warnings, profiling  # noqa
-    from sqlalchemy.testing import config  # noqa
+    from sqlalchemy.testing import config, provision  # noqa
     from sqlalchemy import util  # noqa
 
     warnings.setup_filters()
@@ -281,6 +329,20 @@ def _requirements_opt(opt_str, value, parser):
     _setup_requirements(value)
 
 
+def _set_tag_include(tag):
+    def _do_include_tag(opt_str, value, parser):
+        _include_tag(opt_str, tag, parser)
+
+    return _do_include_tag
+
+
+def _set_tag_exclude(tag):
+    def _do_exclude_tag(opt_str, value, parser):
+        _exclude_tag(opt_str, tag, parser)
+
+    return _do_exclude_tag
+
+
 def _exclude_tag(opt_str, value, parser):
     exclude_tags.add(value.replace("-", "_"))
 
@@ -310,17 +372,33 @@ def _setup_options(opt, file_config):
 
 
 @pre
-def _set_nomemory(opt, file_config):
-    if opt.nomemory:
-        exclude_tags.add("memory_intensive")
+def _register_sqlite_numeric_dialect(opt, file_config):
+    from sqlalchemy.dialects import registry
+
+    registry.register(
+        "sqlite.pysqlite_numeric",
+        "sqlalchemy.dialects.sqlite.pysqlite",
+        "_SQLiteDialect_pysqlite_numeric",
+    )
+    registry.register(
+        "sqlite.pysqlite_dollar",
+        "sqlalchemy.dialects.sqlite.pysqlite",
+        "_SQLiteDialect_pysqlite_dollar",
+    )
 
 
-@pre
-def _monkeypatch_cdecimal(options, file_config):
-    if options.cdecimal:
-        import cdecimal
+@post
+def __ensure_cext(opt, file_config):
+    if os.environ.get("REQUIRE_SQLALCHEMY_CEXT", "0") == "1":
+        from sqlalchemy.util import has_compiled_ext
 
-        sys.modules["decimal"] = cdecimal
+        try:
+            has_compiled_ext(raise_=True)
+        except ImportError as err:
+            raise AssertionError(
+                "REQUIRE_SQLALCHEMY_CEXT is set but can't import the "
+                "cython extensions"
+            ) from err
 
 
 @post
@@ -330,10 +408,16 @@ def _init_symbols(options, file_config):
     config._fixture_functions = _fixture_fn_class()
 
 
+@pre
+def _set_disable_asyncio(opt, file_config):
+    if opt.disable_asyncio:
+        asyncio.ENABLE_ASYNCIO = False
+
+
 @post
 def _engine_uri(options, file_config):
-    from sqlalchemy.testing import config
     from sqlalchemy import testing
+    from sqlalchemy.testing import config
     from sqlalchemy.testing import provision
 
     if options.dburi:
@@ -341,6 +425,8 @@ def _engine_uri(options, file_config):
     else:
         db_urls = []
 
+    extra_drivers = options.dbdriver or []
+
     if options.db:
         for db_token in options.db:
             for db in re.split(r"[,\s]+", db_token):
@@ -356,23 +442,28 @@ def _engine_uri(options, file_config):
         db_urls.append(file_config.get("db", "default"))
 
     config._current = None
-    for db_url in db_urls:
 
-        if options.write_idents and provision.FOLLOWER_IDENT:  # != 'master':
+    expanded_urls = list(provision.generate_db_urls(db_urls, extra_drivers))
+
+    for db_url in expanded_urls:
+        log.info("Adding database URL: %s", db_url)
+
+        if options.write_idents and provision.FOLLOWER_IDENT:
             with open(options.write_idents, "a") as file_:
-                file_.write(provision.FOLLOWER_IDENT + " " + db_url + "\n")
+                file_.write(
+                    f"{provision.FOLLOWER_IDENT} "
+                    f"{db_url.render_as_string(hide_password=False)}\n"
+                )
 
         cfg = provision.setup_config(
             db_url, options, file_config, provision.FOLLOWER_IDENT
         )
-
         if not config._current:
             cfg.set_as_current(cfg, testing)
 
 
 @post
 def _requirements(options, file_config):
-
     requirement_cls = file_config.get("sqla_testing", "requirement_cls")
     _setup_requirements(requirement_cls)
 
@@ -395,70 +486,18 @@ def _setup_requirements(argument):
 
     config.requirements = testing.requires = req_cls()
 
+    config.bootstrapped_as_sqlalchemy = bootstrapped_as_sqlalchemy
+
 
 @post
 def _prep_testing_database(options, file_config):
-    from sqlalchemy.testing import config, util
-    from sqlalchemy.testing.exclusions import against
-    from sqlalchemy import schema, inspect
+    from sqlalchemy.testing import config
 
     if options.dropfirst:
+        from sqlalchemy.testing import provision
+
         for cfg in config.Config.all_configs():
-            e = cfg.db
-            inspector = inspect(e)
-            try:
-                view_names = inspector.get_view_names()
-            except NotImplementedError:
-                pass
-            else:
-                for vname in view_names:
-                    e.execute(
-                        schema._DropView(
-                            schema.Table(vname, schema.MetaData())
-                        )
-                    )
-
-            if config.requirements.schemas.enabled_for_config(cfg):
-                try:
-                    view_names = inspector.get_view_names(schema="test_schema")
-                except NotImplementedError:
-                    pass
-                else:
-                    for vname in view_names:
-                        e.execute(
-                            schema._DropView(
-                                schema.Table(
-                                    vname,
-                                    schema.MetaData(),
-                                    schema="test_schema",
-                                )
-                            )
-                        )
-
-            util.drop_all_tables(e, inspector)
-
-            if config.requirements.schemas.enabled_for_config(cfg):
-                util.drop_all_tables(e, inspector, schema=cfg.test_schema)
-
-            if against(cfg, "postgresql"):
-                from sqlalchemy.dialects import postgresql
-
-                for enum in inspector.get_enums("*"):
-                    e.execute(
-                        postgresql.DropEnumType(
-                            postgresql.ENUM(
-                                name=enum["name"], schema=enum["schema"]
-                            )
-                        )
-                    )
-
-
-@post
-def _reverse_topological(options, file_config):
-    if options.reversetop:
-        from sqlalchemy.orm.util import randomize_unitofwork
-
-        randomize_unitofwork()
+            provision.drop_all_schema_objects(cfg, cfg.db)
 
 
 @post
@@ -474,7 +513,9 @@ def _setup_profiling(options, file_config):
     from sqlalchemy.testing import profiling
 
     profiling._profile_stats = profiling.ProfileStatsFile(
-        file_config.get("sqla_testing", "profile_file")
+        file_config.get("sqla_testing", "profile_file"),
+        sort=options.profilesort,
+        dump=options.profiledump,
     )
 
 
@@ -483,12 +524,6 @@ def want_class(name, cls):
         return False
     elif name.startswith("_"):
         return False
-    elif (
-        config.options.backend_only
-        and not getattr(cls, "__backend__", False)
-        and not getattr(cls, "__sparse_backend__", False)
-    ):
-        return False
     else:
         return True
 
@@ -498,33 +533,13 @@ def want_method(cls, fn):
         return False
     elif fn.__module__ is None:
         return False
-    elif include_tags:
-        return (
-            hasattr(cls, "__tags__")
-            and exclusions.tags(cls.__tags__).include_test(
-                include_tags, exclude_tags
-            )
-        ) or (
-            hasattr(fn, "_sa_exclusion_extend")
-            and fn._sa_exclusion_extend.include_test(
-                include_tags, exclude_tags
-            )
-        )
-    elif exclude_tags and hasattr(cls, "__tags__"):
-        return exclusions.tags(cls.__tags__).include_test(
-            include_tags, exclude_tags
-        )
-    elif exclude_tags and hasattr(fn, "_sa_exclusion_extend"):
-        return fn._sa_exclusion_extend.include_test(include_tags, exclude_tags)
     else:
         return True
 
 
-def generate_sub_tests(cls, module):
-    if getattr(cls, "__backend__", False) or getattr(
-        cls, "__sparse_backend__", False
-    ):
-        sparse = getattr(cls, "__sparse_backend__", False)
+def generate_sub_tests(cls, module, markers):
+    if "backend" in markers or "sparse_backend" in markers:
+        sparse = "sparse_backend" in markers
         for cfg in _possible_configs_for_cls(cls, sparse=sparse):
             orig_name = cls.__name__
 
@@ -546,15 +561,23 @@ def generate_sub_tests(cls, module):
         yield cls
 
 
-def start_test_class(cls):
+def start_test_class_outside_fixtures(cls):
     _do_skips(cls)
     _setup_engine(cls)
 
 
 def stop_test_class(cls):
-    # from sqlalchemy import inspect
-    # assert not inspect(testing.db).get_table_names()
-    engines.testing_reaper._stop_test_ctx()
+    # close sessions, immediate connections, etc.
+    fixtures.stop_test_class_inside_fixtures(cls)
+
+    # close outstanding connection pool connections, dispose of
+    # additional engines
+    engines.testing_reaper.stop_test_class_inside_fixtures()
+
+
+def stop_test_class_outside_fixtures(cls):
+    engines.testing_reaper.stop_test_class_outside_fixtures()
+    provision.stop_test_class_outside_fixtures(config, config.db, cls)
     try:
         if not options.low_connections:
             assertions.global_cleanup_assertions()
@@ -563,23 +586,25 @@ def stop_test_class(cls):
 
 
 def _restore_engine():
-    config._current.reset(testing)
+    if config._current:
+        config._current.reset(testing)
 
 
 def final_process_cleanup():
-    engines.testing_reaper._stop_test_ctx_aggressive()
+    engines.testing_reaper.final_cleanup()
     assertions.global_cleanup_assertions()
     _restore_engine()
 
 
 def _setup_engine(cls):
     if getattr(cls, "__engine_options__", None):
-        eng = engines.testing_engine(options=cls.__engine_options__)
+        opts = dict(cls.__engine_options__)
+        opts["scope"] = "class"
+        eng = engines.testing_engine(options=opts)
         config._current.push_engine(eng, testing)
 
 
 def before_test(test, test_module_name, test_class, test_name):
-
     # format looks like:
     # "test.aaa_profiling.test_compiler.CompileTest.test_update_whereclause"
 
@@ -587,11 +612,16 @@ def before_test(test, test_module_name, test_class, test_name):
 
     id_ = "%s.%s.%s" % (test_module_name, name, test_name)
 
-    profiling._current_test = id_
+    profiling._start_current_test(id_)
 
 
 def after_test(test):
-    engines.testing_reaper._after_test_ctx()
+    fixtures.after_test()
+    engines.testing_reaper.after_test()
+
+
+def after_test_fixtures(test):
+    engines.testing_reaper.after_test_outside_fixtures(test)
 
 
 def _possible_configs_for_cls(cls, reasons=None, sparse=False):
@@ -647,6 +677,7 @@ def _possible_configs_for_cls(cls, reasons=None, sparse=False):
                 all_configs,
                 key=lambda cfg: (
                     cfg.db.name,
+                    cfg.db.driver,
                     cfg.db.dialect.server_version_info,
                 ),
             )
@@ -671,7 +702,8 @@ def _do_skips(cls):
                 )
 
     if not all_configs:
-        msg = "'%s' unsupported on any DB implementation %s%s" % (
+        msg = "'%s.%s' unsupported on any DB implementation %s%s" % (
+            cls.__module__,
             cls.__name__,
             ", ".join(
                 "'%s(%s)+%s'"
@@ -705,7 +737,7 @@ def _setup_config(config_obj, ctx):
     config._current.push(config_obj, testing)
 
 
-class FixtureFunctions(ABC):
+class FixtureFunctions(abc.ABC):
     @abc.abstractmethod
     def skip_test_exception(self, *arg, **kw):
         raise NotImplementedError()
@@ -725,6 +757,14 @@ class FixtureFunctions(ABC):
     def get_current_test_name(self):
         raise NotImplementedError()
 
+    @abc.abstractmethod
+    def mark_base_test_class(self) -> Any:
+        raise NotImplementedError()
+
+    @abc.abstractproperty
+    def add_to_marker(self):
+        raise NotImplementedError()
+
 
 _fixture_fn_class = None
 
