@@ -9,6 +9,8 @@ from sqlalchemy import event
 from sqlalchemy import exc as sa_exc
 from sqlalchemy import ForeignKey
 from sqlalchemy import func
+from sqlalchemy import Identity
+from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import literal_column
 from sqlalchemy import MetaData
@@ -391,6 +393,28 @@ class SynonymTest(QueryTest, AssertsCompiledSQL):
 
 
 class MiscDeprecationsTest(fixtures.TestBase):
+    def test_unloaded_expirable(self, decl_base):
+        class A(decl_base):
+            __tablename__ = "a"
+            id = mapped_column(Integer, Identity(), primary_key=True)
+            x = mapped_column(
+                Integer,
+            )
+            y = mapped_column(Integer, deferred=True)
+
+        decl_base.metadata.create_all(testing.db)
+        with Session(testing.db) as sess:
+            obj = A(x=1, y=2)
+            sess.add(obj)
+            sess.commit()
+
+        with expect_deprecated(
+            "The InstanceState.unloaded_expirable attribute is deprecated.  "
+            "Please use InstanceState.unloaded."
+        ):
+            eq_(inspect(obj).unloaded, {"id", "x", "y"})
+            eq_(inspect(obj).unloaded_expirable, inspect(obj).unloaded)
+
     def test_evaluator_is_private(self):
         with expect_deprecated(
             "Direct use of 'EvaluatorCompiler' is not supported, and this "
@@ -844,6 +868,37 @@ class DeprecatedMapperTest(
 
         self.sql_count_(1, go)
 
+    @testing.variation("prop_type", ["relationship", "col_prop"])
+    def test_prop_replacement_warns(self, prop_type: testing.Variation):
+        users, User = self.tables.users, self.classes.User
+        addresses, Address = self.tables.addresses, self.classes.Address
+
+        m = self.mapper(
+            User,
+            users,
+            properties={
+                "foo": column_property(users.c.name),
+                "addresses": relationship(Address),
+            },
+        )
+        self.mapper(Address, addresses)
+
+        if prop_type.relationship:
+            key = "addresses"
+            new_prop = relationship(Address)
+        elif prop_type.col_prop:
+            key = "foo"
+            new_prop = column_property(users.c.name)
+        else:
+            prop_type.fail()
+
+        with expect_deprecated(
+            f"Property User.{key} on Mapper|User|users being replaced "
+            f"with new property User.{key}; the old property will "
+            "be discarded",
+        ):
+            m.add_property(key, new_prop)
+
 
 class DeprecatedOptionAllTest(OptionsPathTest, _fixtures.FixtureTest):
     run_inserts = "once"
@@ -1151,7 +1206,6 @@ class NonPrimaryRelationshipLoaderTest(_fixtures.FixtureTest):
         self._run_double_test(1)
 
     def test_selectin(self):
-
         users, orders, User, Address, Order, addresses = (
             self.tables.users,
             self.tables.orders,
@@ -1208,7 +1262,6 @@ class NonPrimaryRelationshipLoaderTest(_fixtures.FixtureTest):
         self._run_double_test(4)
 
     def test_subqueryload(self):
-
         users, orders, User, Address, Order, addresses = (
             self.tables.users,
             self.tables.orders,
